// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/20AOF16C500KR/driver/irq.c $
// $Rev: 46532 $
// $Date: 2019-09-17 14:10:41 -0500 (Tue, 17 Sep 2019) $

// 20AOF16C500KR: Device Driver: source file

#include "main.h"



//*****************************************************************************
void dev_irq_isr_local_handler(dev_data_t* dev)
{
	static const struct
	{
		s32	irq;
		s32	wait;
	} list[]	=
	{
		{ AOF16C500KR_IRQ_AUTO_CAL_DONE,	AOF16C500KR_WAIT_GSC_AUTO_CAL_DN	},
		{ AOF16C500KR_IRQ_DIO0_L2H,			AOF16C500KR_WAIT_GSC_DIO0_L2H		},
		{ AOF16C500KR_IRQ_BUF_THR_H2L,		AOF16C500KR_WAIT_GSC_BUF_THR_H2L	},
		{ AOF16C500KR_IRQ_BUF_THR_L2H,		AOF16C500KR_WAIT_GSC_BUF_THR_L2H	},
		{ AOF16C500KR_IRQ_LOAD_READY_H2L,	AOF16C500KR_WAIT_GSC_LD_RDY_H2L		},
		{ AOF16C500KR_IRQ_LOAD_READY_L2H,	AOF16C500KR_WAIT_GSC_LD_RDY_L2H		},
		{ AOF16C500KR_IRQ_BURST_READY,		AOF16C500KR_WAIT_GSC_BURST_READY	},
		{ AOF16C500KR_IRQ_BUF_ERROR,		AOF16C500KR_WAIT_GSC_BUF_ERROR		},
		{ -1, -1 }
	};

	int	i;
	u32	irq;
	u32	psr;

	psr	= os_reg_mem_rx_u32(NULL, dev->vaddr.gsc_psr_32);
	irq	= psr & (psr >> 16) & 0xFFFF;

	if (irq == 0)
	{
		// We don't know the source of the interrupt.
		gsc_wait_resume_irq_main(dev, GSC_WAIT_MAIN_SPURIOUS);
	}
	else
	{
		// Clear the interrupt.
		psr	= psr | 0xFFFF0000;

		for (i = 0; list[i].irq >= 0; i++)
		{
			if (irq & list[i].irq)
			{
				irq	^= list[i].irq;
				psr	&= ~(list[i].irq << 16);	// Service the interrupt.
				gsc_wait_resume_irq_gsc(dev, list[i].wait);
			}
		}

		if (irq)
		{
			// Firmware has interrupts the driver doesn't support.
			gsc_wait_resume_irq_main(dev, GSC_WAIT_MAIN_SPURIOUS);
			psr	&= ~irq;	// Disable the interrupt(s)
		}

		// Apply the changes.
		os_reg_mem_tx_u32(NULL, dev->vaddr.gsc_psr_32, psr);
	}
}



//*****************************************************************************
int dev_irq_create(dev_data_t* dev)
{
	int	ret;

	os_reg_mem_tx_u32(dev, dev->vaddr.gsc_psr_32, 0);
	ret	= gsc_irq_create(dev);
	return(ret);
}



//*****************************************************************************
void dev_irq_destroy(dev_data_t* dev)
{
	if (dev->vaddr.gsc_psr_32)
		os_reg_mem_tx_u32(dev, dev->vaddr.gsc_psr_32, 0);

	gsc_irq_destroy(dev);
}


