// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/20AOF16C500KR/samples/irq/perform.c $
// $Rev: 52367 $
// $Date: 2023-01-12 10:14:47 -0600 (Thu, 12 Jan 2023) $

// 20AOF16C500KR: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _auto_cal(int fd)
{
	int	errs	= 0;
	int	ret;
	s32	val;

	val		= AOF16C500KR_IRQ_AUTO_CAL_DONE;
	ret		= aof16c500kr_ioctl(fd, AOF16C500KR_IOCTL_IRQ_SEL, &val);
	errs	+= ret ? 1 : 0;

	ret		= aof16c500kr_ioctl(fd, AOF16C500KR_IOCTL_AUTO_CALIBRATE, NULL);
	errs	+= ret ? 1 : 0;
	return(errs);
}



//*****************************************************************************
static int _irq_auto_cal_done_test(int fd)
{
	int	errs	= 0;

	gsc_label("Auto-Calibrate Done");
	errs	+= aof16c500kr_initialize(fd, -1, 0);
	errs	+= irq_wait_test(fd, AOF16C500KR_WAIT_GSC_AUTO_CAL_DN, _auto_cal);
	errs	+= aof16c500kr_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _dio0_l2h(int fd)
{
	int	errs	= 0;
	int	ret;
	s32	val;

	val		= AOF16C500KR_DIO_DIR_OUT_D7_D0;
	ret		= aof16c500kr_ioctl(fd, AOF16C500KR_IOCTL_DIO_DIR_OUT, &val);
	errs	+= ret ? 1 : 0;

	val		= 0x00;
	ret		= aof16c500kr_ioctl(fd, AOF16C500KR_IOCTL_DIO_WRITE, &val);
	errs	+= ret ? 1 : 0;

	val		= AOF16C500KR_IRQ_DIO0_L2H;
	ret		= aof16c500kr_ioctl(fd, AOF16C500KR_IOCTL_IRQ_SEL, &val);
	errs	+= ret ? 1 : 0;

	val		= 0x01;
	ret		= aof16c500kr_ioctl(fd, AOF16C500KR_IOCTL_DIO_WRITE, &val);
	errs	+= ret ? 1 : 0;
	return(errs);
}



//*****************************************************************************
static int _irq_dio0_l2h_test(int fd)
{
	int	errs	= 0;

	gsc_label("DIO 0 Low->High");
	errs	+= aof16c500kr_initialize(fd, -1, 0);
	errs	+= irq_wait_test(fd, AOF16C500KR_WAIT_GSC_DIO0_L2H, _dio0_l2h);
	errs	+= aof16c500kr_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _buf_thr_h2l(int fd)
{
	u32	buf[128]	= { 0 };
	int	errs		= 0;
	int	qty;
	int	ret;
	s32	val;

	qty		= aof16c500kr_write(fd, buf, sizeof(buf));

	if (qty != sizeof(buf))
	{
		errs++;
		printf(	"FAIL <---  (%d. write: requested %ld samples, sent %ld)\n",
				__LINE__,
				(long) sizeof(buf),
				(long) qty);
	}

	val		= 10;
	ret		= aof16c500kr_ioctl(fd, AOF16C500KR_IOCTL_BUF_THRESH, &val);
	errs	+= ret ? 1 : 0;

	val		= AOF16C500KR_IRQ_BUF_THR_H2L;
	ret		= aof16c500kr_ioctl(fd, AOF16C500KR_IOCTL_IRQ_SEL, &val);
	errs	+= ret ? 1 : 0;

	val		= 10000;
	ret		= aof16c500kr_ioctl(fd, AOF16C500KR_IOCTL_BUF_THRESH, &val);
	errs	+= ret ? 1 : 0;
	return(errs);
}



//*****************************************************************************
static int _irq_buf_thr_h2l_test(int fd)
{
	int	errs	= 0;

	gsc_label("Buffer Threshold Hi->Low");
	errs	+= aof16c500kr_initialize(fd, -1, 0);
	errs	+= irq_wait_test(fd, AOF16C500KR_WAIT_GSC_BUF_THR_H2L, _buf_thr_h2l);
	errs	+= aof16c500kr_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _buf_thr_l2h(int fd)
{
	u32	buf[128]	= { 0 };
	int	errs		= 0;
	int	qty;
	int	ret;
	s32	val;

	qty		= aof16c500kr_write(fd, buf, sizeof(buf));

	if (qty != sizeof(buf))
	{
		errs++;
		printf(	"FAIL <---  (%d. write: requested %ld samples, sent %ld)\n",
				__LINE__,
				(long) sizeof(buf),
				(long) qty);
	}

	val		= 10000;
	ret		= aof16c500kr_ioctl(fd, AOF16C500KR_IOCTL_BUF_THRESH, &val);
	errs	+= ret ? 1 : 0;

	val		= AOF16C500KR_IRQ_BUF_THR_L2H;
	ret		= aof16c500kr_ioctl(fd, AOF16C500KR_IOCTL_IRQ_SEL, &val);
	errs	+= ret ? 1 : 0;

	val		= 10;
	ret		= aof16c500kr_ioctl(fd, AOF16C500KR_IOCTL_BUF_THRESH, &val);
	errs	+= ret ? 1 : 0;
	return(errs);
}



//*****************************************************************************
static int _irq_buf_thr_l2h_test(int fd)
{
	int	errs	= 0;

	gsc_label("Buffer Threshold Low->Hi");
	errs	+= aof16c500kr_initialize(fd, -1, 0);
	errs	+= irq_wait_test(fd, AOF16C500KR_WAIT_GSC_BUF_THR_L2H, _buf_thr_l2h);
	errs	+= aof16c500kr_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _load_ready_h2l(int fd)
{
	int	errs	= 0;
	int	ret;
	s32	val;

	val		= AOF16C500KR_BUF_MODE_OPEN;
	ret		= aof16c500kr_ioctl(fd, AOF16C500KR_IOCTL_BUF_MODE, &val);
	errs	+= ret ? 1 : 0;

	val		= AOF16C500KR_IRQ_LOAD_READY_H2L;
	ret		= aof16c500kr_ioctl(fd, AOF16C500KR_IOCTL_IRQ_SEL, &val);
	errs	+= ret ? 1 : 0;

	val		= AOF16C500KR_BUF_MODE_CIRC;
	ret		= aof16c500kr_ioctl(fd, AOF16C500KR_IOCTL_BUF_MODE, &val);
	errs	+= ret ? 1 : 0;
	return(errs);
}



//*****************************************************************************
static int _irq_load_ready_h2l_test(int fd)
{
	int	errs	= 0;

	gsc_label("Load Ready Hi->Low");
	errs	+= aof16c500kr_initialize(fd, -1, 0);
	errs	+= irq_wait_test(fd, AOF16C500KR_WAIT_GSC_LD_RDY_H2L, _load_ready_h2l);
	errs	+= aof16c500kr_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _load_ready_l2h(int fd)
{
	int	errs	= 0;
	int	ret;
	s32	val;

	val		= AOF16C500KR_BUF_MODE_CIRC;
	ret		= aof16c500kr_ioctl(fd, AOF16C500KR_IOCTL_BUF_MODE, &val);
	errs	+= ret ? 1 : 0;

	val		= AOF16C500KR_IRQ_LOAD_READY_L2H;
	ret		= aof16c500kr_ioctl(fd, AOF16C500KR_IOCTL_IRQ_SEL, &val);
	errs	+= ret ? 1 : 0;

	val		= AOF16C500KR_BUF_MODE_OPEN;
	ret		= aof16c500kr_ioctl(fd, AOF16C500KR_IOCTL_BUF_MODE, &val);
	errs	+= ret ? 1 : 0;
	return(errs);
}



//*****************************************************************************
static int _irq_load_ready_l2h_test(int fd)
{
	int	errs	= 0;

	gsc_label("Load Ready Low->Hi");
	errs	+= aof16c500kr_initialize(fd, -1, 0);
	errs	+= irq_wait_test(fd, AOF16C500KR_WAIT_GSC_LD_RDY_L2H, _load_ready_l2h);
	errs	+= aof16c500kr_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _burst_ready(int fd)
{
	u32	buf[128]	= { 0 };
	int	errs		= 0;
	int	qty;
	int	ret;
	s32	val;

	val		= AOF16C500KR_IRQ_BURST_READY;
	ret		= aof16c500kr_ioctl(fd, AOF16C500KR_IOCTL_IRQ_SEL, &val);
	errs	+= ret ? 1 : 0;

	val		= AOF16C500KR_BURST_ENABLE_YES;
	ret		= aof16c500kr_ioctl(fd, AOF16C500KR_IOCTL_BURST_ENABLE, &val);
	errs	+= ret ? 1 : 0;

	qty		= aof16c500kr_write(fd, buf, sizeof(buf));

	if (qty != sizeof(buf))
	{
		errs++;
		printf(	"FAIL <---  (%d. write: requested %ld samples, sent %ld)\n",
				__LINE__,
				(long) sizeof(buf),
				(long) qty);
	}

	val		= AOF16C500KR_CLOCK_ENABLE_YES;
	ret		= aof16c500kr_ioctl(fd, AOF16C500KR_IOCTL_CLOCK_ENABLE, &val);
	errs	+= ret ? 1 : 0;
	return(errs);
}



//*****************************************************************************
static int _irq_burst_ready_test(int fd)
{
	int	errs	= 0;

	gsc_label("Burst Ready");
	errs	+= aof16c500kr_initialize(fd, -1, 0);
	errs	+= irq_wait_test(fd, AOF16C500KR_WAIT_GSC_BURST_READY, _burst_ready);
	errs	+= aof16c500kr_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _buf_error(int fd)
{
	int	errs	= 0;
	int	ret;
	s32	val;

	val		= AOF16C500KR_BUF_MODE_CIRC;
	ret		= aof16c500kr_ioctl(fd, AOF16C500KR_IOCTL_BUF_MODE, &val);
	errs	+= ret ? 1 : 0;

	val		= AOF16C500KR_IRQ_BUF_ERROR;
	ret		= aof16c500kr_ioctl(fd, AOF16C500KR_IOCTL_IRQ_SEL, &val);
	errs	+= ret ? 1 : 0;

	errs	+= aof16c500kr_reg_write(fd, -1, 0, AOF16C500KR_GSC_AOBR, 0);
	return(errs);
}



//*****************************************************************************
static int _irq_buf_error_test(int fd)
{
	int	errs	= 0;

	gsc_label("Buffer Error");
	errs	+= aof16c500kr_initialize(fd, -1, 0);
	errs	+= irq_wait_test(fd, AOF16C500KR_WAIT_GSC_BUF_ERROR, _buf_error);
	errs	+= aof16c500kr_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int	errs	= 0;

	gsc_label("IRQ");
	printf("\n");
	gsc_label_level_inc();

	errs	+= _irq_auto_cal_done_test(args->fd);
	errs	+= _irq_dio0_l2h_test(args->fd);
	errs	+= _irq_buf_thr_h2l_test(args->fd);
	errs	+= _irq_buf_thr_l2h_test(args->fd);
	errs	+= _irq_load_ready_h2l_test(args->fd);
	errs	+= _irq_load_ready_l2h_test(args->fd);
	errs	+= _irq_burst_ready_test(args->fd);
	errs	+= _irq_buf_error_test(args->fd);

	gsc_label_level_dec();
	return(errs);
}


