// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/20AOF16C500KR/samples/signals/perform.c $
// $Rev: 52370 $
// $Date: 2023-01-12 10:15:45 -0600 (Thu, 12 Jan 2023) $

// 20AOF16C500KR: Sample Application: source file

#include "main.h"



// variables ******************************************************************

static	u32		_data[128 * 1024];
static	long	_loops;	// NOT USED HERE. Number of times to send a block of data.
static	long	_qty;	// Number of samples to send per block.



//*****************************************************************************
static void _init_data(int chans, long rate, int seconds)
{
	#define	PIE		(3.1415962)

	u32*		dest;
	int			i;
	long		period_len;	// The length of the pattern in samples.
	long		period_tot;	// Pattern samples total for all channels.
	long		period;		// Pattern length in x units.
	long		samp;
	float		samp_dat;	// Total sample size of the data buffer.
	float		samp_sec;	// Samples Per Second for all active channels
	float		samp_tot;	// Samples Total for all active channels
	const char*	units;

	gsc_label("Initializing Data");
	memset(_data, 0, sizeof(_data));
	samp_sec	= ((float) rate) * chans;
	samp_tot	= ((float) rate) * chans * seconds;
	samp_dat	= sizeof(_data) / 4;

	period_len	= (rate <= 100) ? rate				//      10-      100 ->    10-   100
				: (rate <= 1000) ? (rate / 2)		//     101-    1,000 ->    50-   500
				: (rate <= 10000) ? (rate / 5)		//   1,001-   10,000 ->   200- 2,000
				: (rate <= 100000) ? (rate / 10)	//  10,001-  100,000 -> 1,000-10,000
				: (rate / 100);						// 100,001-1,000,000 -> 1,000-10,000
	period_tot	= period_len * chans;
	period		= (long) (1000000000. / rate * period_len);
	units		= "ns";

	if ((period % 1000) == 0)
	{
		period	/= 1000;
		units	= "us";
	}

	if ((period % 1000) == 0)
	{
		period	/= 1000;
		units	= "ms";
	}

	if (samp_tot <= SIZEOF_ARRAY(_data))
	{
		// We can get the entire image in the buffer.
		_loops	= 1;
		_qty	= (long) samp_tot;
	}
	else if (samp_sec <= SIZEOF_ARRAY(_data))
	{
		// We can get one second of the image in the buffer.
		_loops	= seconds;
		_qty	= (long) samp_sec;
	}
	else if (period_tot <= SIZEOF_ARRAY(_data))
	{
		// We can get one period of the image in the buffer.
		_loops	= seconds * (samp_sec / period_tot);
		_qty	= period_tot;
	}
	else
	{
		// We can get only a fraction of a period of the image in the buffer.
		_qty	= (samp_dat / chans) * chans;
		_loops	= (samp_tot + _qty - 1) / _qty;
	}

	// Initialize the data buffer.
	// The data is encoded in OFFSET BINARY format.
	dest	= _data;;

	for (i = 0, samp = 0; i < _qty; i += chans, samp++)
	{
		samp	= (samp < period_len) ? samp : 0;

		// 1st channel: sawtooth wave
		*dest++	= 0xFFFFFL - (0xFFFFFL * samp / (period_len - 1));
	}

	dest[-1]	|= 0x100000;
	printf("DONE   (pattern length: %ld samples, %ld %s)\n", period_len, period, units);
}



//*****************************************************************************
static int _write_data(int fd)
{
	int	errs	= 0;
	int	qty;

	gsc_label("Writing Data");

	qty	= aof16c500kr_write(fd, _data, _qty * 4);

	if (qty >= 0)
		printf("Done\n");
	else
		errs++;

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int	errs	= 0;
	s32	fsamp	= 25000L;
	s32	ndiv;

	errs	+= aof16c500kr_initialize		(args->fd, -1, 1);
	errs	+= aof16c500kr_channel_sel		(args->fd, -1, 1, 0x0001, NULL);
	errs	+= aof16c500kr_buf_mode			(args->fd, -1, 1, AOF16C500KR_BUF_MODE_OPEN, NULL);
	errs	+= aof16c500kr_clock_enable		(args->fd, -1, 1, AOF16C500KR_CLOCK_ENABLE_NO, NULL);
	_init_data(1, fsamp, 1);
	errs	+= _write_data(args->fd);
//	errs	+= aof16c500kr_burst_enable		(args->fd, -1, 1, AOF16C500KR_BURST_ENABLE_YES, NULL);
	errs	+= aof16c500kr_fsamp_ao_compute	(args->fd, -1, 1, fsamp, &ndiv, NULL);
	errs	+= aof16c500kr_buf_mode			(args->fd, -1, 1, AOF16C500KR_BUF_MODE_CIRC, NULL);

	gsc_label("Ext Clk Out");
	printf("\n");
	gsc_label_level_inc();
	errs	+= aof16c500kr_gen_a_ndiv		(args->fd, -1, 1, ndiv, NULL);
	errs	+= aof16c500kr_clock_src		(args->fd, -1, 1, AOF16C500KR_CLOCK_SRC_RAG, NULL);
	errs	+= aof16c500kr_gen_a_enable		(args->fd, -1, 1, AOF16C500KR_GEN_ENABLE_YES, NULL);
	errs	+= aof16c500kr_clock_enable		(args->fd, -1, 1, AOF16C500KR_CLOCK_ENABLE_YES, NULL);
	errs	+= aof16c500kr_xcvr_type		(args->fd, -1, 1, AOF16C500KR_XCVR_TYPE_LVDS, NULL);
	errs	+= aof16c500kr_xcvr_output		(args->fd, -1, 1, AOF16C500KR_XCVR_OUTPUT_ENABLE, NULL);
	gsc_label_level_dec();

#if 1
	gsc_label("Ext Sync Out");
	printf("\n");
	gsc_label_level_inc();
//	errs	+= aof16c500kr_burst_trig_src	(args->fd, -1, 1, AOF16C500KR_BURST_TRIG_SRC_RBG, NULL);
	errs	+= aof16c500kr_gen_b_ndiv		(args->fd, -1, 1, ndiv * 5, NULL);
//	errs	+= aof16c500kr_gen_b_enable		(args->fd, -1, 1, AOF16C500KR_GEN_ENABLE_YES, NULL);
	gsc_label_level_dec();
#endif

	gsc_label("Waiting");
	os_sleep_ms(args->seconds * 1000);
	printf("Done\n");

	errs	+= aof16c500kr_initialize		(args->fd, -1, 1);

	return(errs);
}



