// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/20AOF16C500KR/samples/txrate/perform.c $
// $Rev: 52371 $
// $Date: 2023-01-12 10:16:02 -0600 (Thu, 12 Jan 2023) $

// 20AOF16C500KR: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	_1M	(1024L * 1024L)



// variables ******************************************************************

static u32	_tx[_1M];



//*****************************************************************************
static int _rate_test(const args_t* args, int seconds, long send)
{
	os_time_ns_t	begin;
	int				errs		= 0;
	os_time_ns_t	minimum;
	long long		ms;
	os_time_ns_t	now			= { 0 };
	long			sent;
	long			sent_acc	= 0;
	long			sent_mb		= 0;
	float			total;

	gsc_label("Writing");

	os_time_get_ns(&begin);
	minimum.tv_sec	= begin.tv_sec + seconds;
	minimum.tv_nsec	= begin.tv_nsec;

	memset(_tx, 0x01, sizeof(_tx));

	for (;;)
	{
		sent	= aof16c500kr_write(args->fd, _tx, send);

		if (sent < 0)
		{
			errs++;
			printf("FAIL <--- (write, error %ld)\n", sent);
			break;
		}

		os_time_get_ns(&now);
		sent_acc	+= sent;

		if (sent_acc >= _1M)
		{
			for (; sent_acc >= _1M;)
			{
				sent_acc	-= _1M;
				sent_mb++;
			}

			if (((now.tv_sec > minimum.tv_sec) ||
				((now.tv_sec == minimum.tv_sec) && (now.tv_nsec >= minimum.tv_nsec))) &&
				(sent_mb >= args->tx_mb))
			{
				break;
			}
		}
	}

	if (errs == 0)
	{
		total	= (float) sent_mb * _1M + sent_acc;
		ms		= ((long long) now.tv_sec * 1000 + now.tv_nsec / 1000000)
				- ((long long) begin.tv_sec * 1000 + begin.tv_nsec / 1000000);
		printf(	"PASS  (");
		gsc_label_long_comma((long) total);
		printf(" Bytes, %lld.%03lld Seconds, ", ms / 1000, ms % 1000);
		gsc_label_long_comma((long) (total / ms * 1000 / 4));
		printf(" S/S)\n");
	}

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int		errs	= 0;
	int		seconds	= 5;
	long	send	= sizeof(_tx);

	errs	+= aof16c500kr_config_ao			(args->fd, -1, 1, 500000L);
	errs	+= aof16c500kr_tx_io_mode			(args->fd, -1, 1, args->io_mode, NULL);
	errs	+= aof16c500kr_fsamp_ao_report_all	(args->fd, -1, 1, NULL);

	gsc_label ("Write Rate");
	printf("(%d second minimum, %ld MB minimum)\n", (int) seconds, (long) args->tx_mb);
	gsc_label_level_inc();

	errs	+= _rate_test(args, seconds, send);

	gsc_label_level_dec();
	return(errs);
}


