// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/20AOF16C500KR/utils/util_auto_cal_sts.c $
// $Rev: 46528 $
// $Date: 2019-09-17 14:05:47 -0500 (Tue, 17 Sep 2019) $

// 20AOF16C500KR: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	aof16c500kr_auto_cal_sts
*
*	Purpose:
*
*		Provide a visual wrapper for the AOF16C500KR_IOCTL_AUTO_CAL_STS service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		get		The current setting is recorded here, if not NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int	aof16c500kr_auto_cal_sts(int fd, int index, int verbose, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr		= buf;
	int			ret;
	s32			set;

	if (verbose)
		gsc_label_index("Auto-Cal Status", index);

	ret		= aof16c500kr_ioctl(fd, AOF16C500KR_IOCTL_AUTO_CAL_STS, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			sprintf(buf, "INVALID: %ld", (long) set);
			break;

		case AOF16C500KR_AUTO_CAL_STS_ACTIVE:

			ptr	= "Active";
			break;

		case AOF16C500KR_AUTO_CAL_STS_FAIL:

			ptr	= "Fail";
			break;

		case AOF16C500KR_AUTO_CAL_STS_PASS:

			ptr	= "Pass";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


