// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/20AOF16C500KR/utils/util_buf_status.c $
// $Rev: 46528 $
// $Date: 2019-09-17 14:05:47 -0500 (Tue, 17 Sep 2019) $

// 20AOF16C500KR: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	aof16c500kr_buf_status
*
*	Purpose:
*
*		Provide a visual wrapper for the AOF16C500KR_IOCTL_BUF_STATUS service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int aof16c500kr_buf_status(int fd, int index, int verbose, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr		= buf;
	int			ret;
	s32			set;

	if (verbose)
		gsc_label_index("Buffer Status", index);

	ret		= aof16c500kr_ioctl(fd, AOF16C500KR_IOCTL_BUF_STATUS, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			sprintf(buf, "INVALID: %ld", (long) set);
			break;

		case AOF16C500KR_BUF_STATUS_EMPTY:

			ptr	= "Empty";
			break;

		case AOF16C500KR_BUF_STATUS_AE:

			ptr	= "Almost Empty";
			break;

		case AOF16C500KR_BUF_STATUS_AF:

			ptr	= "Almost Full";
			break;

		case AOF16C500KR_BUF_STATUS_FULL:

			ptr	= "Full";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


