// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/20AOF16C500KR/utils/util_config_ao.c $
// $Rev: 52363 $
// $Date: 2023-01-12 10:12:20 -0600 (Thu, 12 Jan 2023) $

// 20AOF16C500KR: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	aof16c500kr_config_ao
*
*	Purpose:
*
*		Configure the given device using common defaults.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		fsamp	This is the desired Fsamp rate, of -1 to use the default.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int	aof16c500kr_config_ao(int fd, int index, int verbose, s32 fsamp)
{
	int	errs	= 0;
	s32	mask;
	s32	ndiv;
	s32	range;

	if (verbose)
	{
		gsc_label_index("Output Configuration", index);
		printf("\n");
		gsc_label_level_inc();
	}

	errs	+= aof16c500kr_initialize			(fd, index, verbose);
	errs	+= aof16c500kr_tx_io_mode			(fd, index, verbose, GSC_IO_MODE_PIO,					NULL);
	errs	+= aof16c500kr_tx_io_ovr_dat		(fd, index, verbose, AOF16C500KR_TX_IO_OVR_DAT_CHECK,	NULL);
	errs	+= aof16c500kr_tx_io_ovr_frm		(fd, index, verbose, AOF16C500KR_TX_IO_OVR_DAT_CHECK,	NULL);
	errs	+= aof16c500kr_tx_io_timeout		(fd, index, verbose, 30,								NULL);
	errs	+= aof16c500kr_buf_mode				(fd, index, verbose, AOF16C500KR_BUF_MODE_OPEN,			NULL);
	errs	+= aof16c500kr_buf_thresh			(fd, index, verbose, 200000L,							NULL);
	errs	+= aof16c500kr_output_mode			(fd, index, verbose, AOF16C500KR_OUTPUT_MODE_SIM,		NULL);
	errs	+= aof16c500kr_filter				(fd, index, verbose, AOF16C500KR_FILTER_DISABLE,		NULL);

	range	= AOF16C500KR_QUERY_RANGE;
	errs	+= aof16c500kr_ioctl_util			(fd, -1, 0, AOF16C500KR_IOCTL_QUERY, &range);
	range	= (range == AOF16C500KR_RANGE_6_3) ? AOF16C500KR_RANGE_6 : AOF16C500KR_RANGE_10;
	errs	+= aof16c500kr_range				(fd, index, verbose, range,								NULL);

	errs	+= aof16c500kr_irq_sel				(fd, index, verbose, 0,									NULL);
	errs	+= aof16c500kr_burst_enable			(fd, index, verbose, AOF16C500KR_BURST_ENABLE_NO,		NULL);
	errs	+= aof16c500kr_burst_trig_src		(fd, index, verbose, AOF16C500KR_BURST_TRIG_SRC_RAG,	NULL);

	mask	= AOF16C500KR_QUERY_CHANNEL_MASK;
	errs	+= aof16c500kr_ioctl_util			(fd, -1, 0, AOF16C500KR_IOCTL_QUERY, &mask);
	errs	+= aof16c500kr_channel_sel			(fd, index, verbose, mask,								NULL);

	errs	+= aof16c500kr_data_format			(fd, index, verbose, AOF16C500KR_DATA_FORMAT_OFF_BIN,	NULL);
	errs	+= aof16c500kr_ground_sense			(fd, index, verbose, AOF16C500KR_GROUND_SENSE_DISABLE,	NULL);
	errs	+= aof16c500kr_dio_dir_out			(fd, index, verbose, AOF16C500KR_DIO_DIR_OUT_NONE,		NULL);
	errs	+= aof16c500kr_clock_enable			(fd, index, verbose, AOF16C500KR_CLOCK_ENABLE_YES,		NULL);
	errs	+= aof16c500kr_fsamp_ao_compute		(fd, index, verbose, fsamp, &ndiv,						NULL);
	errs	+= aof16c500kr_clock_src			(fd, index, verbose, AOF16C500KR_CLOCK_SRC_RAG,			NULL);
	errs	+= aof16c500kr_gen_a_ndiv			(fd, index, verbose, ndiv,								NULL);
	errs	+= aof16c500kr_gen_b_ndiv			(fd, index, verbose, ndiv,								NULL);
	errs	+= aof16c500kr_gen_a_enable			(fd, index, verbose, AOF16C500KR_GEN_ENABLE_YES,		NULL);
	errs	+= aof16c500kr_gen_b_enable			(fd, index, verbose, AOF16C500KR_GEN_ENABLE_NO,			NULL);
	errs	+= aof16c500kr_fsamp_ao_report_all	(fd, index, verbose,									NULL);

	errs	+= aof16c500kr_xcvr_output			(fd, index, verbose, AOF16C500KR_XCVR_OUTPUT_DISABLE,	NULL);
	errs	+= aof16c500kr_xcvr_type			(fd, index, verbose, AOF16C500KR_XCVR_TYPE_LVDS,		NULL);
	errs	+= aof16c500kr_xcvr_pol				(fd, index, verbose, AOF16C500KR_XCVR_POL_DEF,			NULL);
	errs	+= aof16c500kr_led					(fd, index, verbose, AOF16C500KR_LED_ON,				NULL);
//	errs	+= aof16c500kr_auto_calibrate		(fd, index, verbose);
	errs	+= aof16c500kr_buf_clear			(fd, index, verbose);
	errs	+= aof16c500kr_buf_over_data		(fd, index, verbose, AOF16C500KR_BUF_OVER_DATA_CLR,		NULL);
	errs	+= aof16c500kr_buf_over_frame		(fd, index, verbose, AOF16C500KR_BUF_OVER_FRAME_CLR,	NULL);

	if (verbose)
		gsc_label_level_dec();

	return(errs);
}



