// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/20AOF16C500KR/utils/util_id.c $
// $Rev: 52363 $
// $Date: 2023-01-12 10:12:20 -0600 (Thu, 12 Jan 2023) $

// 20AOF16C500KR: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	_id_device_pci
*
*	Purpose:
*
*		Identify the device using the PCI registers.
*
*	Arguments:
*
*		fd		The handle used to access the device.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

static int _id_device_pci(int fd)
{
	int	errs;
	u32	reg;

	gsc_label("Vendor ID");
	errs	= aof16c500kr_reg_read(fd, -1, 0, GSC_PCI_9656_VIDR, &reg);
	printf("0x%04lX      ", (long) reg);

	if (reg == 0x10B5)
	{
		printf("(PLX)\n");
	}
	else
	{
		errs++;
		printf("(UNKNOWN) FAIL <---\n");
	}

	gsc_label("Device ID");
	errs	+= aof16c500kr_reg_read(fd, -1, 0, GSC_PCI_9656_DIDR, &reg);
	printf("0x%04lX      ", (long) reg);

	if (reg == 0x9656)
	{
		printf("(PCI9656)\n");
	}
	else
	{
		errs++;
		printf("(UNKNOWN) FAIL <---\n");
	}

	gsc_label("Sub Vendor ID");
	errs	+= aof16c500kr_reg_read(fd, -1, 0, GSC_PCI_9656_SVID, &reg);
	printf("0x%04lX      ", (long) reg);

	if (reg == 0x10B5)
	{
		printf("(PLX)\n");
	}
	else
	{
		errs++;
		printf("(UNKNOWN) FAIL <---\n");
	}

	gsc_label("Subsystem ID");
	errs	+= aof16c500kr_reg_read(fd, -1, 0, GSC_PCI_9656_SID, &reg);
	printf("0x%04lX      ", (long) reg);

	if (reg == 0x3491)
	{
		printf("(20AOF16C500KR)\n");
	}
	else
	{
		errs++;
		printf("(UNKNOWN) FAIL <---\n");
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	aof16c500kr_id_device
*
*	Purpose:
*
*		Identify the device.
*
*	Arguments:
*
*		fd		The handle to use to access the device.
*
*		index	The index of the device or -1 if we don't care.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int aof16c500kr_id_device(int fd, int index, int verbose)
{
	const char*	desc		= NULL;
	s32			device_type;
	int			errs;

	if (verbose)
		gsc_label_index("Device", index);

	errs	= aof16c500kr_query(fd, -1, 0, AOF16C500KR_QUERY_DEVICE_TYPE, &device_type);

	if (errs == 0)
	{
		switch (device_type)
		{
			default:

				errs	= 1;

				if (verbose)
				{
					printf(	"FAIL <---  (unexpected device type: %ld)",
							(long) device_type);
				}

				break;

			case GSC_DEV_TYPE_20AOF16C500KR:

				desc	= "20AOF16C500KR";
				break;
		}
	}

	if (verbose)
	{
		printf("%s\n", desc ? desc : "");
		gsc_label_level_inc();
	}

	errs	+= _id_device_pci(fd);

	if (verbose)
		gsc_label_level_dec();

	return(errs);
}



//*****************************************************************************
int aof16c500kr_id_driver(void)
{
	int	errs;

	errs	= os_id_driver(aof16c500kr_open, aof16c500kr_read, aof16c500kr_close);
	return(errs);
}


