// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/20AOF16C500KR/utils/util_query.c $
// $Rev: 52363 $
// $Date: 2023-01-12 10:12:20 -0600 (Thu, 12 Jan 2023) $

// 20AOF16C500KR: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	aof16c500kr_query
*
*	Purpose:
*
*		Provide a visual wrapper for the AOF16C500KR_IOCTL_QUERY service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the query option to access.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int aof16c500kr_query(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128]	= "";
	int			errs		= 0;
	const char*	ptr;
	s32			query		= set;
	int			ret;

	switch (query)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Query Error (#%ld)", (long) query);
			break;

		case AOF16C500KR_QUERY_AO_CONFIG:		ptr	= "AO Configuration";	break;
		case AOF16C500KR_QUERY_AUTO_CAL_MS:		ptr	= "Auto-Cal Duration";	break;
		case AOF16C500KR_QUERY_CHANNEL_MASK:	ptr	= "Channel Mask";		break;
		case AOF16C500KR_QUERY_CHANNEL_MAX:		ptr	= "Max Channels";		break;
		case AOF16C500KR_QUERY_CHANNEL_QTY:		ptr	= "Installed Channels";	break;
		case AOF16C500KR_QUERY_COUNT:			ptr	= "Query Options";		break;
		case AOF16C500KR_QUERY_CUSTOM_FEAT:		ptr	= "Custom Feature";		break;
		case AOF16C500KR_QUERY_DEVICE_TYPE:		ptr	= "Device Type";		break;
		case AOF16C500KR_QUERY_FGEN_MAX:		ptr	= "Fgen Max";			break;
		case AOF16C500KR_QUERY_FGEN_MIN:		ptr	= "Fgen Min";			break;
		case AOF16C500KR_QUERY_FIFO_SIZE:		ptr	= "FIFO Size";			break;
		case AOF16C500KR_QUERY_FILTER:			ptr	= "Filter";				break;
		case AOF16C500KR_QUERY_FREF_DEFAULT:	ptr	= "Fref Default";		break;
		case AOF16C500KR_QUERY_FSAMP_MAX:		ptr	= "Fsamp Max";			break;
		case AOF16C500KR_QUERY_FSAMP_MIN:		ptr	= "Fsamp Min";			break;
		case AOF16C500KR_QUERY_INIT_MS:			ptr	= "Init. Duration";		break;
		case AOF16C500KR_QUERY_NDIV_MAX:		ptr	= "Ndiv Max";			break;
		case AOF16C500KR_QUERY_NDIV_MIN:		ptr	= "Ndiv Min";			break;
		case AOF16C500KR_QUERY_RANGE:			ptr	= "Voltage Range";		break;
	}

	if (verbose)
		gsc_label_index(ptr, index);

	ret		= aof16c500kr_ioctl(fd, AOF16C500KR_IOCTL_QUERY, &set);
	errs	+= ret ? 1 : 0;

	switch (query)
	{
		default:

			errs++;
			sprintf(buf, "%ld", (long) set);
			break;

		case AOF16C500KR_QUERY_CHANNEL_MAX:
		case AOF16C500KR_QUERY_CHANNEL_QTY:


			gsc_label_long_comma_buf(set, buf);
			strcat(buf, "  (20-bit resolution)");
			break;

		case AOF16C500KR_QUERY_COUNT:
		case AOF16C500KR_QUERY_NDIV_MAX:
		case AOF16C500KR_QUERY_NDIV_MIN:

			gsc_label_long_comma_buf(set, buf);
			break;

		case AOF16C500KR_QUERY_CHANNEL_MASK:

			sprintf(buf, "0x%lX", (long) set);
			break;

		case AOF16C500KR_QUERY_FGEN_MAX:
		case AOF16C500KR_QUERY_FGEN_MIN:
		case AOF16C500KR_QUERY_FREF_DEFAULT:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " Hz");
			break;

		case AOF16C500KR_QUERY_AUTO_CAL_MS:
		case AOF16C500KR_QUERY_INIT_MS:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " ms");
			break;

		case AOF16C500KR_QUERY_FSAMP_MAX:
		case AOF16C500KR_QUERY_FSAMP_MIN:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " S/S");
			break;

		case AOF16C500KR_QUERY_FIFO_SIZE:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " Samples Deep");

			if ((set) && ((set % 1024) == 0))
			{
				strcat(buf, " (");
				gsc_label_long_comma_buf(set / 1024, buf + strlen(buf));
				strcat(buf, "K)");
			}

			break;

		case AOF16C500KR_QUERY_DEVICE_TYPE:

			if (set == GSC_DEV_TYPE_20AOF16C500KR)
			{
				strcpy(buf, "20AOF16C500KR");
			}
			else
			{
				errs++;
				sprintf(buf, "INVALID: %ld", (long) set);
			}

			break;

		case AOF16C500KR_QUERY_AO_CONFIG:

			switch (set)
			{
				default:

					errs++;
					sprintf(buf, "Invalid option: %ld", (long) set);
					break;

				case AOF16C500KR_AO_CONFIG_DIFF:	strcpy(buf, "Differential");	break;
				case AOF16C500KR_AO_CONFIG_SE:		strcpy(buf, "Single Ended");	break;
			}

			break;

		case AOF16C500KR_QUERY_CUSTOM_FEAT:

			switch (set)
			{
				default:

					errs++;
					sprintf(buf, "Invalid option: %ld", (long) set);
					break;

				case 0:	strcpy(buf, "None");	break;
			}

			break;

		case AOF16C500KR_QUERY_FILTER:

			switch (set)
			{
				default:

					errs++;
					sprintf(buf, "Invalid option: %ld", (long) set);
					break;

				case AOF16C500KR_FILTER_140K:	strcpy(buf, "Type F1: DC-140KHz, Elliptic, 8th Order");	break;
			}

			break;

		case AOF16C500KR_QUERY_RANGE:

			switch (set)
			{
				default:

					errs++;
					sprintf(buf, "Invalid option: %ld", (long) set);
					break;

				case AOF16C500KR_RANGE_6_3:		strcpy(buf, "+-6 Volts, +-3 Volts");	break;
				case AOF16C500KR_RANGE_10_5:	strcpy(buf, "+-10 Volts, +-5 Volts");	break;
			}

			break;
	}

	if (verbose)
	{
		if (errs)
			printf("FAIL <---  (%s)\n", buf);
		else
			printf("%s\n", buf);
	}

	if (get)
		get[0]	= set;

	return(errs);
}


