// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/20AOF16C500KR/utils/util_reg.c $
// $Rev: 52363 $
// $Date: 2023-01-12 10:12:20 -0600 (Thu, 12 Jan 2023) $

// 20AOF16C500KR: Utilities: source file

#include "main.h"



// macros *********************************************************************

#define	_GSC_REG(a)				"GSC " #a, AOF16C500KR_GSC_##a
#define	SHOW_FIELD(b,e,eol,l,d)	gsc_reg_field_show(width+10,23, value,(b),(e),(eol),(l),(d))
#define	SIZEOF_ARRAY(a)			(sizeof((a)) / sizeof((a)[0]))



//*****************************************************************************
static int _bcr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	disable[]	= { "Disable",			"Enable"		};
	static const char*	fail[]		= { "Fail",				"Pass"			};
	static const char*	idle[]		= { "Idle",				"Active"		};
	static const char*	off[]		= { "Off",				"On"			};
	static const char*	sequenc[]	= { "Sequential",		"Simultaneous"	};
	static const char*	ttl[]		= { "TTL",				"LVDS"			};
	static const char*	twos[]		= { "2s Compliment",	"Offset Binary"	};

	SHOW_FIELD( 4,  0, 1, NULL,		"Reserved"				);
	SHOW_FIELD( 5,  5, 1, ttl,		"Transceiver Type"		);
	SHOW_FIELD( 6,  6, 1, disable,	"Remote Ground Sense"	);
	SHOW_FIELD(17,  7, 1, NULL,		"Reserved"				);
	SHOW_FIELD(18, 18, 1, sequenc,	"Output Timing"			);
	SHOW_FIELD(19, 19, 1, off,		"Panel LED"				);
	SHOW_FIELD(20, 20, 1, idle,		"SW Clock"				);
	SHOW_FIELD(21, 21, 1, disable,	"Rate-A Generator"		);
	SHOW_FIELD(22, 22, 1, disable,	"Rate-B Generator"		);
	SHOW_FIELD(23, 23, 1, idle,		"SW Trigger"			);
	SHOW_FIELD(24, 24, 1, disable,	"Output Filters"		);
	SHOW_FIELD(25, 25, 1, twos,		"Data Format"			);
	SHOW_FIELD(27, 26, 1, NULL,		"Reserved"				);
	SHOW_FIELD(28, 28, 1, idle,		"Auto-Calibration"		);
	SHOW_FIELD(29, 29, 1, fail,		"Auto-Cal Status"		);
	SHOW_FIELD(30, 30, 1, NULL,		"Reserved"				);
	SHOW_FIELD(31, 31, 1, idle,		"Initialize"		);
	return(0);
}



//*****************************************************************************
static int _diopr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	input[]	= { "Input",	"Output"	};

	SHOW_FIELD( 3,  0, 1, NULL,		"DIO D0-D3"				);
	SHOW_FIELD( 6,  4, 1, NULL,		"Reserved"				);
	SHOW_FIELD( 7,  7, 1, input,	"DIO D0-D3 Direction"	);
	SHOW_FIELD(11,  8, 1, NULL,		"DIO D4-D7"				);
	SHOW_FIELD(14, 12, 1, NULL,		"Reserved"				);
	SHOW_FIELD(15, 15, 1, input,	"DIO D4-D7 Direction"	);
	SHOW_FIELD(31, 16, 1, NULL,		"Reserved"				);
	return(0);
}



//*****************************************************************************
static int _psr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	disable[]	= { "Disable",	"Enable"	};
	static const char*	idle[]		= { "Idle",		"Active"	};

	SHOW_FIELD( 0,  0, 1, disable,	"Auto-Calibration"		);
	SHOW_FIELD( 7,  1, 1, NULL,		"Reserved"				);
	SHOW_FIELD( 8,  8, 1, disable,	"DIO D0 Low to High"	);
	SHOW_FIELD( 9,  9, 1, disable,	"Threshold Hi to Low"	);
	SHOW_FIELD(10, 10, 1, disable,	"Threshold Low to Hi"	);
	SHOW_FIELD(11, 11, 1, disable,	"Load Ready Hi to Low"	);
	SHOW_FIELD(12, 12, 1, disable,	"Load Ready Low to Hi"	);
	SHOW_FIELD(13, 13, 1, disable,	"Burst Ready"			);
	SHOW_FIELD(14, 14, 1, disable,	"Data/Frame Overflow"	);
	SHOW_FIELD(15, 15, 1, NULL,		"Reserved"				);
	SHOW_FIELD(16, 16, 1, idle,		"Auto-Calibration"		);
	SHOW_FIELD(23, 17, 1, NULL,		"Reserved"				);
	SHOW_FIELD(24, 24, 1, idle,		"DIO D0 Low to High"	);
	SHOW_FIELD(25, 25, 1, idle,		"Threshold Hi to Low"	);
	SHOW_FIELD(26, 26, 1, idle,		"Threshold Low to Hi"	);
	SHOW_FIELD(27, 27, 1, idle,		"Load Ready Hi to Low"	);
	SHOW_FIELD(28, 28, 1, idle,		"Load Ready Low to Hi"	);
	SHOW_FIELD(29, 29, 1, idle,		"Burst Ready"			);
	SHOW_FIELD(30, 30, 1, idle,		"Data/Frame Overflow"	);
	SHOW_FIELD(31, 31, 1, NULL,		"Reserved"				);
	return(0);
}



//*****************************************************************************
static int _acr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	clock[]	=
	{
		"49.152MHz",
		"RESERVED",
		"RESERVED",
		"RESERVED"
	};

	static const char*	filter[]	=
	{
		"Type F1: DC-140KHz, Elliptic, 8th Order",
		"RESERVED",
		"RESERVED",
		"RESERVED"
	};

	static const char*	range[]	=
	{
		"+-6V, +-3V",
		"+-10V, +-5V",
		"RESERVED",
		"RESERVED"
	};

	static const char*	channels[]	= { "16", "12", "8", "4" };
	static const char*	diff[]		= { "Differential", "Single Ended" };

	long	v;

	SHOW_FIELD(15,  0, 0, NULL,		"Firmware Version"		);
	v	= GSC_FIELD_DECODE(value, 15, 0);
	printf("%04lX\n", v);

	SHOW_FIELD(17, 16, 1, channels,	"Output Channels"		);
	SHOW_FIELD(19, 18, 1, clock,	"Master Clock"			);
	SHOW_FIELD(21, 20, 1, filter,	"Output Filter"			);
	SHOW_FIELD(23, 22, 1, range,	"Output Range"			);
	SHOW_FIELD(24, 24, 1, diff,		"Output Configuration"	);
	SHOW_FIELD(27, 25, 1, NULL,		"Custom Features"		);
	SHOW_FIELD(31, 28, 1, NULL,		"Reserved"				);
	return(0);
}



//*****************************************************************************
static int _boor_decode(int fd, int supported, u32 value, int width)
{
	static const char*	clear[]		= { "Clear",		"Set"		};
	static const char*	disable[]	= { "Disabled",		"Enabled"	};
	static const char*	idle[]		= { "Idle",			"Active"	};
	static const char*	circ[]		= { "Open",			"Circular"	};
	static const char*	no[]		= { "No",			"Yes"		};
	static const char*	nready[]	= { "Not Ready",	"Ready"		};

	SHOW_FIELD( 4,  0, 1, NULL,		"Reserved"			);
	SHOW_FIELD( 5,  5, 1, disable,	"Output Clocking"	);
	SHOW_FIELD( 6,  6, 1, nready,	"Clock Ready"		);
	SHOW_FIELD( 7,  7, 1, idle,		"SW Clock"			);
	SHOW_FIELD( 8,  8, 1, circ,		"Buffer Mode"		);
	SHOW_FIELD( 9,  9, 1, idle,		"Load Request"		);
	SHOW_FIELD(10, 10, 1, nready,	"Load Ready"		);
	SHOW_FIELD(11, 11, 1, idle,		"Clear Buffer"		);
	SHOW_FIELD(12, 12, 1, no,		"Buffer Empty"		);
	SHOW_FIELD(13, 13, 1, clear,	"Buffer Threshold"	);
	SHOW_FIELD(14, 14, 1, NULL,		"Reserved"			);
	SHOW_FIELD(15, 15, 1, no,		"Buffer Full"		);
	SHOW_FIELD(16, 16, 1, no,		"Buffer Overflow"	);
	SHOW_FIELD(17, 17, 1, no,		"Frame Overflow"	);
	SHOW_FIELD(18, 18, 1, nready,	"Burst Ready"		);
	SHOW_FIELD(19, 19, 1, disable,	"Output Bursting"	);
	SHOW_FIELD(20, 20, 1, idle,		"SW Trigger"		);
	SHOW_FIELD(31, 21, 1, NULL,		"Reserved"			);
	return(0);
}



//*****************************************************************************
static int _obtr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	clear[]	= { "Clear",	"Set"	};

	SHOW_FIELD(18,  0, 1, NULL,		"Threshold"			);
	SHOW_FIELD(19, 19, 1, NULL,		"Reserved"			);
	SHOW_FIELD(20, 20, 1, clear,	"Threshold Flag"	);
	SHOW_FIELD(31, 21, 1, NULL,		"Reserved"			);
	return(0);
}



//*****************************************************************************
static int _obsr_decode(int fd, int supported, u32 value, int width)
{
	SHOW_FIELD(18,  0, 1, NULL,	"Fill Level"	);
	SHOW_FIELD(31, 19, 1, NULL,	"Reserved"		);
	return(0);
}



//*****************************************************************************
static int _rxgr_decode(int fd, int supported, u32 value, int width)
{
	SHOW_FIELD(23,  0, 1, NULL,	"Ndiv"		);
	SHOW_FIELD(31, 24, 1, NULL,	"Reserved"	);
	return(0);
}



//*****************************************************************************
static int _ocr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	rag[]		=
	{
		"Rate-A Generator",
		"Rate-B Generator",
		"Analog Output External Clock input",
		"RESERVED"
	};

	static const char*	_3[]		= { "+-3V",		"+-6V"		};
	static const char*	_5[]		= { "+-5V",		"+-10V"		};
	static const char*	disable[]	= { "Disable",	"Enable"	};

	s32	channels	= 16;
	int	errs		= 0;
	s32	range;

	errs	+= aof16c500kr_query(fd, -1, 0, AOF16C500KR_QUERY_CHANNEL_QTY, &channels);
	errs	+= aof16c500kr_query(fd, -1, 0, AOF16C500KR_QUERY_RANGE, &range);

	SHOW_FIELD( 0,  0, 1, disable,		"Output 0"	);
	SHOW_FIELD( 1,  1, 1, disable,		"Output 1"	);
	SHOW_FIELD( 2,  2, 1, disable,		"Output 2"	);
	SHOW_FIELD( 3,  3, 1, disable,		"Output 3"	);

	if (channels > 4)
	{
		SHOW_FIELD( 4,  4, 1, disable,	"Output 4"	);
		SHOW_FIELD( 5,  5, 1, disable,	"Output 5"	);
		SHOW_FIELD( 6,  6, 1, disable,	"Output 6"	);
		SHOW_FIELD( 7,  7, 1, disable,	"Output 7"	);
	}
	else
	{
		SHOW_FIELD( 7,  4, 1, NULL,		"Reserved"	);
	}

	if (channels > 8)
	{
		SHOW_FIELD( 8,  8, 1, disable,	"Output 8"	);
		SHOW_FIELD( 9,  9, 1, disable,	"Output 9"	);
		SHOW_FIELD(10, 10, 1, disable,	"Output 10"	);
		SHOW_FIELD(11, 11, 1, disable,	"Output 11"	);
	}
	else
	{
		SHOW_FIELD(11,  8, 1, NULL,		"Reserved"	);
	}

	if (channels > 12)
	{
		SHOW_FIELD(12, 12, 1, disable,	"Output 12"	);
		SHOW_FIELD(13, 13, 1, disable,	"Output 13"	);
		SHOW_FIELD(14, 14, 1, disable,	"Output 14"	);
		SHOW_FIELD(15, 15, 1, disable,	"Output 15"	);
	}
	else
	{
		SHOW_FIELD(15, 12, 1, NULL,		"Reserved"	);
	}

	if (range == AOF16C500KR_RANGE_6_3)
		SHOW_FIELD(16, 16, 1, _3,		"Output Range"			);
	else
		SHOW_FIELD(16, 16, 1, _5,		"Output Range"			);

	SHOW_FIELD(17, 17, 1, disable,		"Clock/Trigger Out"		);
	SHOW_FIELD(19, 18, 1, rag,			"Clock Source"			);
	SHOW_FIELD(21, 20, 1, rag,			"Trigger Source"		);
	SHOW_FIELD(22, 22, 1, NULL,			"Reserved"				);
	SHOW_FIELD(23, 23, 1, disable,		"Invert Clk/Trig I/O"	);
	SHOW_FIELD(31, 24, 1, NULL,			"Reserved"				);
	return(errs);
}



// variables ******************************************************************

static gsc_reg_def_t	_gsc[]	=
{
	// name	reg			err	value	ask_support	decode				desc
	{ _GSC_REG(BCR),	0,	0,		0,			_bcr_decode,		"Board Control Register"				},
	{ _GSC_REG(DIOPR),	0,	0,		0,			_diopr_decode,		"Digital I/O Port Register"				},
	{ _GSC_REG(PSR),	0,	0,		0,			_psr_decode,		"Primary Status Register"				},
	{ _GSC_REG(ACR),	0,	0,		0,			_acr_decode,		"Assembly Configuration Register"		},
	{ _GSC_REG(AVR),	0,	0,		0,			NULL,				"Auto Cal Values Register"				},
	{ _GSC_REG(BOOR),	0,	0,		0,			_boor_decode,		"Buffered Output Operations Register"	},
	{ _GSC_REG(OBTR),	0,	0,		0,			_obtr_decode,		"Output Buffer Threshold Register"		},
	{ _GSC_REG(OBSR),	0,	0,		0,			_obsr_decode,		"Output Buffer Size Register"			},
	{ _GSC_REG(AOBR),	0,	0,		0,			NULL,				"Analog Output Buffer Register"			},
	{ _GSC_REG(RAGR),	0,	0,		0,			_rxgr_decode,		"Rate-A Generator Register"				},
	{ _GSC_REG(RBGR),	0,	0,		0,			_rxgr_decode,		"Rate-B Generator Register"				},
	{ _GSC_REG(OCR),	0,	0,		0,			_ocr_decode,		"Output Configuration Register"			},
	{ NULL }
};



//*****************************************************************************
static const gsc_reg_def_t* _find_reg(u32 reg, const gsc_reg_def_t* list)
{
	const gsc_reg_def_t*	def	= NULL;
	int						i;

	for (i = 0; list[i].name; i++)
	{
		if (reg == list[i].reg)
		{
			def	= &list[i];
			break;
		}
	}

	return(def);
}



/******************************************************************************
*
*	Function:	aof16c500kr_reg_get_def_id
*
*	Purpose:
*
*		Retrieve the register definition structure given the register id.
*
*	Arguments:
*
*		reg		The id of the register to access.
*
*	Returned:
*
*		NULL	The register id wasn't found.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* aof16c500kr_reg_get_def_id(u32 reg)
{
	const gsc_reg_def_t*	def;

	def	= _find_reg(reg, _gsc);
	return(def);
}



/******************************************************************************
*
*	Function:	aof16c500kr_reg_get_def_index
*
*	Purpose:
*
*		Retrieve the register definition structure based on an index.
*
*	Arguments:
*
*		index	The index of the register to access.
*
*	Returned:
*
*		NULL	The index doesn't correspond to a known register.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* aof16c500kr_reg_get_def_index(int index)
{
	const gsc_reg_def_t*	def;

	if (index < 0)
		def	= NULL;
	else if (index >= (SIZEOF_ARRAY(_gsc) - 1))
		def	= NULL;
	else
		def	= &_gsc[index];

	return(def);
}



/******************************************************************************
*
*	Function:	aof16c500kr_reg_get_desc
*
*	Purpose:
*
*		Retrieve the description of the specified register.
*
*	Arguments:
*
*		reg		The register whose description is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* aof16c500kr_reg_get_desc(u32 reg)
{
	const gsc_reg_def_t*	def;
	const char*				desc;

	def	= _find_reg(reg, _gsc);

	if (def)
		desc	= def->desc;
	else
		desc	= "UNKNOWN";

	return(desc);
}



/******************************************************************************
*
*	Function:	aof16c500kr_reg_get_name
*
*	Purpose:
*
*		Retrieve the name of the specified register.
*
*	Arguments:
*
*		reg		The register whose name is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* aof16c500kr_reg_get_name(u32 reg)
{
	const gsc_reg_def_t*	def;
	const char*				name;

	def	= _find_reg(reg, _gsc);

	if (def)
		name	= def->name;
	else
		name	= "UNKNOWN";

	return(name);
}



/******************************************************************************
*
*	Function:	aof16c500kr_reg_list
*
*	Purpose:
*
*		List the GSC registers and their values.
*
*	Arguments:
*
*		fd		The handle to access the device.
*
*		detail	List the register details?
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int aof16c500kr_reg_list(int fd, int detail)
{
	int	errs;

	errs	= gsc_reg_list(fd, _gsc, detail, aof16c500kr_reg_read);
	return(errs);
}



