// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/20AOF16C500KR/utils/util_xcvr_pol.c $
// $Rev: 46528 $
// $Date: 2019-09-17 14:05:47 -0500 (Tue, 17 Sep 2019) $

// 20AOF16C500KR: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	aof16c500kr_xcvr_pol
*
*	Purpose:
*
*		Provide a visual wrapper for the AOF16C500KR_IOCTL_XCVR_POL service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int aof16c500kr_xcvr_pol(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr		= buf;
	int			ret;

	if (verbose)
		gsc_label_index("Transceiver Polarity", index);

	ret		= aof16c500kr_ioctl(fd, AOF16C500KR_IOCTL_XCVR_POL, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case AOF16C500KR_XCVR_POL_DEF:

			ptr	= "Default";
			break;

		case AOF16C500KR_XCVR_POL_INV:

			ptr	= "Inverted";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


